'use strict';
var nothingness = 1, nGain = 0.01; //todo: outsource into another file

var canvas, gl, positionAttributeLocation, positionBuffer, matBuffer, vpBuffer, sizeBuffer, distBuffer, fgBuffer;

function createShader(gl, type, source) {
    var shader = gl.createShader(type);
    gl.shaderSource(shader, source);
    gl.compileShader(shader);
    var success = gl.getShaderParameter(shader, gl.COMPILE_STATUS);
    if (success) {
        return shader;
    }

    console.log(gl.getShaderInfoLog(shader));
    gl.deleteShader(shader);
}

function createProgram(gl, vertexShader, fragmentShader) {
    var program = gl.createProgram();
    gl.attachShader(program, vertexShader);
    gl.attachShader(program, fragmentShader);
    gl.linkProgram(program);
    var success = gl.getProgramParameter(program, gl.LINK_STATUS);
    if (success) {
        return program;
    }

    console.log(gl.getProgramInfoLog(program));
    gl.deleteProgram(program);
}

function main() {
    // Get A WebGL context
    canvas = document.querySelector("#c");
    gl = canvas.getContext("webgl");
    if (!gl) {
        return;
    }

    var vertexShaderSource = document.querySelector("#vertex-shader-2d").text;
    var fragmentShaderSource = document.querySelector("#fragment-shader-2d").text;

    var vertexShader = createShader(gl, gl.VERTEX_SHADER, vertexShaderSource);
    var fragmentShader = createShader(gl, gl.FRAGMENT_SHADER, fragmentShaderSource);

    var program = createProgram(gl, vertexShader, fragmentShader);

    //clean this msss
    positionAttributeLocation = gl.getAttribLocation(program, "a_position");
    matBuffer = gl.getUniformLocation(program, "mat_thing");
    sizeBuffer = gl.getUniformLocation(program, "size_thing");
    distBuffer = gl.getUniformLocation(program, "dist_thing");
    fgBuffer = gl.getUniformLocation(program, "is_fg");
    vpBuffer = gl.getUniformLocation(program, "mvp_thing");

    positionBuffer = gl.createBuffer();

    gl.bindBuffer(gl.ARRAY_BUFFER, positionBuffer);

    gl.useProgram(program);

    gl.enable(gl.DEPTH_TEST);

    gl.depthFunc(gl.LEQUAL);
}

function renderThing() {
    twgl.resizeCanvasToDisplaySize(gl.canvas);

    gl.viewport(0, 0, gl.canvas.width, gl.canvas.height);

    gl.clearColor(0, 0, 0, 0);
    gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);

    var ratio = gl.canvas.width / gl.canvas.height;
    var ratioA = Math.max(ratio, 1);
    var ratioB = ratioA / ratio;


    // Tell the attribute how to get data out of positionBuffer (ARRAY_BUFFER)
    var size = 3;          // 2 components per iteration
    var type = gl.FLOAT;   // the data is 32bit floats
    var normalize = false; // don't normalize the data
    var stride = 0;        // 0 = move forward size * sizeof(type) each iteration to get the next position
    var offset = 0;        // start at the beginning of the buffer
    gl.vertexAttribPointer(
        positionAttributeLocation, size, type, normalize, stride, offset);

    var positions = [
        -1, -1, -1,
        -1, 1, -1,
        1, 1, -1,
        -1, -1, -1,
        1, -1, -1,
        1, 1, -1
    ];
    gl.enableVertexAttribArray(positionAttributeLocation);

    gl.bindBuffer(gl.ARRAY_BUFFER, positionBuffer);

    gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(positions), gl.STATIC_DRAW);

    gl.uniformMatrix4fv(vpBuffer, false, new Float32Array([
        256, 0, 0, 0,
        0, 256, 0, 0,
        0, 0, 256 / 4, 0,
        0, 0, 0, 1
    ]));

    gl.uniformMatrix4fv(matBuffer, false, new Float32Array(MDN.perspectiveMatrix(Math.PI * 100 / 180, ratio, 0.01, 1000)));

    hud.textContent = `${Math.floor(nothingness * 100) / 100} nothings; ${Math.floor(nGain * 60 * 100) / 100} nil/sec`

    gl.uniform1f(sizeBuffer, -Math.log(nothingness));
    gl.uniform1f(distBuffer, 0.1 + 3 * nGain / (nGain + nothingness));
    gl.uniform1i(fgBuffer, false);

    var primitiveType = gl.TRIANGLES;
    var offset = 0;
    var count = 6;
    gl.drawArrays(primitiveType, offset, count);

    positions = MDN.createCubeData(1, 1, 1);
    gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(positions), gl.STATIC_DRAW);

    let vp = MDN.rotateYMatrix(nothingness);
    vp = MDN.multiplyMatrices(MDN.translateMatrix(0, 0, -3), vp)

    gl.uniformMatrix4fv(vpBuffer, false, new Float32Array(vp));

    gl.uniform1i(fgBuffer, true);
    count = positions.length / 3;
    gl.drawArrays(primitiveType, offset, count);

    requestAnimationFrame(renderThing);
}
let hud = document.querySelector('.hud');

main();
requestAnimationFrame(renderThing)

let clicking = false;

setInterval(function () {

    nothingness += nGain;
}, 1000 / 60)

document.body.onclick = async function () {
    if (clicking) return;
    clicking = true;
    let oldNGain = nGain;
    nGain += 0.05;
    while (nGain > oldNGain) {
        nGain -= 0.005;
        await new Promise((res) => setTimeout(res, 1000 / 60))
    }
    nGain = oldNGain;
    clicking = false;
}